@@ -30,31 +30,55 @@
 
 using namespace std;
 
+namespace {
+
+string host_header(const string &host, const string &port)
+{
+	if (port == "http" || atoi(port.c_str()) == 80) {
+		return host;
+	} else {
+		return host + ":" + port;
+	}
+}
+
+}  // namespace
+
 extern ServerPool *servers;
 
-HTTPInput::HTTPInput(const string &url)
+HTTPInput::HTTPInput(const string &url, Input::Encoding encoding)
 	: state(NOT_CONNECTED),
 	  url(url),
+	  encoding(encoding),
 	  has_metacube_header(false),
-	  sock(-1)
+	  sock(-1),
+	  num_connection_attempts(0),
+	  suppress_logging(false)
 {
 	pthread_mutex_init(&stats_mutex, NULL);
 	stats.url = url;
 	stats.bytes_received = 0;
 	stats.data_bytes_received = 0;
 	stats.connect_time = -1;
+
+	last_verbose_connection.tv_sec = -3600;
+	last_verbose_connection.tv_nsec = 0;
 }
 
 HTTPInput::HTTPInput(const InputProto &serialized)
 	: state(State(serialized.state())),
 	  url(serialized.url()),
+	  encoding(serialized.is_metacube_encoded() ?
+	           Input::INPUT_ENCODING_METACUBE :
+	           Input::INPUT_ENCODING_RAW),
 	  request(serialized.request()),
 	  request_bytes_sent(serialized.request_bytes_sent()),
 	  response(serialized.response()),
 	  http_header(serialized.http_header()),
 	  stream_header(serialized.stream_header()),
 	  has_metacube_header(serialized.has_metacube_header()),
-	  sock(serialized.sock())
+	  sock(serialized.sock()),
+	  num_connection_attempts(0),
+	  suppress_logging(false)
 {
 	pending_data.resize(serialized.pending_data().size());
 	memcpy(&pending_data[0], serialized.pending_data().data(), serialized.pending_data().size());
@@ -71,6 +95,9 @@ HTTPInput::HTTPInput(const InputProto &serialized)
 	} else {
 		stats.connect_time = time(NULL);
 	}
+
+	last_verbose_connection.tv_sec = -3600;
+	last_verbose_connection.tv_nsec = 0;
 }
 
 void HTTPInput::close_socket()
@@ -100,6 +127,12 @@ InputProto HTTPInput::serialize() const
 	serialized.set_bytes_received(stats.bytes_received);
 	serialized.set_data_bytes_received(stats.data_bytes_received);
 	serialized.set_connect_time(stats.connect_time);
+	if (encoding == Input::INPUT_ENCODING_METACUBE) {
+		serialized.set_is_metacube_encoded(true);
+	} else {
+		assert(encoding == Input::INPUT_ENCODING_RAW);
+		serialized.set_is_metacube_encoded(false);
+	}
 	return serialized;
 }
 
@@ -108,8 +141,10 @@ int HTTPInput::lookup_and_connect(const string &host, const string &port)
 	addrinfo *ai;
 	int err = getaddrinfo(host.c_str(), port.c_str(), NULL, &ai);
 	if (err != 0) {
-		log(WARNING, "[%s] Lookup of '%s' failed (%s).",
-			url.c_str(), host.c_str(), gai_strerror(err));
+		if (!suppress_logging) {
+			log(WARNING, "[%s] Lookup of '%s' failed (%s).",
+				url.c_str(), host.c_str(), gai_strerror(err));
+		}
 		return -1;
 	}
 
@@ -177,8 +212,10 @@ int HTTPInput::lookup_and_connect(const string &host, const string &port)
 	}
 
 	// Give the last one as error.
-	log(WARNING, "[%s] Connect to '%s' failed (%s)",
-	        url.c_str(), host.c_str(), strerror(errno));
+	if (!suppress_logging) {
+		log(WARNING, "[%s] Connect to '%s' failed (%s)",
+			url.c_str(), host.c_str(), strerror(errno));
+	}
 	freeaddrinfo(base_ai);
 	return -1;
 }
@@ -187,21 +224,27 @@ bool HTTPInput::parse_response(const string &request)
 {
 	vector<string> lines = split_lines(response);
 	if (lines.empty()) {
-		log(WARNING, "[%s] Empty HTTP response from input.", url.c_str());
+		if (!suppress_logging) {
+			log(WARNING, "[%s] Empty HTTP response from input.", url.c_str());
+		}
 		return false;
 	}
 
 	vector<string> first_line_tokens = split_tokens(lines[0]);
 	if (first_line_tokens.size() < 2) {
-		log(WARNING, "[%s] Malformed response line '%s' from input.",
-		        url.c_str(), lines[0].c_str());
+		if (!suppress_logging) {
+			log(WARNING, "[%s] Malformed response line '%s' from input.",
+				url.c_str(), lines[0].c_str());
+		}
 		return false;
 	}
 
 	int response = atoi(first_line_tokens[1].c_str());
 	if (response != 200) {
-		log(WARNING, "[%s] Non-200 response '%s' from input.",
-			url.c_str(), lines[0].c_str());
+		if (!suppress_logging) {
+			log(WARNING, "[%s] Non-200 response '%s' from input.",
+				url.c_str(), lines[0].c_str());
+		}
 		return false;
 	}
 
@@ -273,7 +316,9 @@ void HTTPInput::do_work()
 			timespec elapsed = clock_diff(last_activity, now);
 			if (elapsed.tv_sec >= timeout_secs) {
 				// Timeout!
-				log(ERROR, "[%s] Timeout after %d seconds, closing.", url.c_str(), elapsed.tv_sec);
+				if (!suppress_logging) {
+					log(ERROR, "[%s] Timeout after %d seconds, closing.", url.c_str(), elapsed.tv_sec);
+				}
 				state = CLOSING_SOCKET;
 				continue;
 			}
@@ -308,7 +353,9 @@ void HTTPInput::do_work()
 			{
 				string protocol, user;  // Thrown away.
 				if (!parse_url(url, &protocol, &user, &host, &port, &path)) {
-					log(WARNING, "[%s] Failed to parse URL '%s'", url.c_str(), url.c_str());
+					if (!suppress_logging) {
+						log(WARNING, "[%s] Failed to parse URL '%s'", url.c_str(), url.c_str());
+					}
 					break;
 				}
 
@@ -320,6 +367,24 @@ void HTTPInput::do_work()
 				}
 			}
 
+			if (suppress_logging) {
+				// See if there's more than one minute since last time we made a connection
+				// with logging enabled. If so, turn it on again.
+				timespec now;
+				int err = clock_gettime(CLOCK_MONOTONIC_COARSE, &now);
+				assert(err != -1);
+
+				double elapsed = now.tv_sec - last_verbose_connection.tv_sec +
+					1e-9 * (now.tv_nsec - last_verbose_connection.tv_nsec);
+				if (elapsed > 60.0) {
+					suppress_logging = false;
+				}
+			}
+			if (!suppress_logging) {
+				int err = clock_gettime(CLOCK_MONOTONIC_COARSE, &last_verbose_connection);
+				assert(err != -1);
+			}
+			++num_connection_attempts;
 			sock = lookup_and_connect(host, port);
 			if (sock != -1) {
 				// Yay, successful connect. Try to set it as nonblocking.
@@ -329,7 +394,7 @@ void HTTPInput::do_work()
 					state = CLOSING_SOCKET;
 				} else {
 					state = SENDING_REQUEST;
-					request = "GET " + path + " HTTP/1.0\r\nUser-Agent: cubemap\r\n\r\n";
+					request = "GET " + path + " HTTP/1.0\r\nHost: " + host_header(host, port) + "\r\nUser-Agent: cubemap\r\n\r\n";
 					request_bytes_sent = 0;
 				}
 
@@ -376,8 +441,10 @@ void HTTPInput::do_work()
 
 			if (ret == 0) {
 				// This really shouldn't happen...
-				log(ERROR, "[%s] Socket unexpectedly closed while reading header",
-				           url.c_str());
+				if (!suppress_logging) {
+					log(ERROR, "[%s] Socket unexpectedly closed while reading header",
+						   url.c_str());
+				}
 				state = CLOSING_SOCKET;
 				continue;
 			}
@@ -385,7 +452,9 @@ void HTTPInput::do_work()
 			RequestParseStatus status = wait_for_double_newline(&response, buf, ret);
 			
 			if (status == RP_OUT_OF_SPACE) {
-				log(WARNING, "[%s] Server sent overlong HTTP response!", url.c_str());
+				if (!suppress_logging) {
+					log(WARNING, "[%s] Server sent overlong HTTP response!", url.c_str());
+				}
 				state = CLOSING_SOCKET;
 				continue;
 			} else if (status == RP_NOT_FINISHED_YET) {
@@ -412,8 +481,16 @@ void HTTPInput::do_work()
 				process_data(&extra_data[0], extra_data.size());
 			}
 
-			log(INFO, "[%s] Connected to '%s', receiving data.",
-				   url.c_str(), url.c_str());
+			if (!suppress_logging) {
+				if (encoding == Input::INPUT_ENCODING_RAW) {
+					log(INFO, "[%s] Connected to '%s', receiving raw data.",
+						   url.c_str(), url.c_str());
+				} else {
+					assert(encoding == Input::INPUT_ENCODING_METACUBE);
+					log(INFO, "[%s] Connected to '%s', receiving data.",
+						   url.c_str(), url.c_str());
+				}
+			}
 			state = RECEIVING_DATA;
 			break;
 		}
@@ -433,12 +510,28 @@ void HTTPInput::do_work()
 
 			if (ret == 0) {
 				// This really shouldn't happen...
-				log(ERROR, "[%s] Socket unexpectedly closed while reading data",
-				           url.c_str());
+				if (!suppress_logging) {
+					log(ERROR, "[%s] Socket unexpectedly closed while reading data",
+						   url.c_str());
+				}
 				state = CLOSING_SOCKET;
 				continue;
 			}
 
+			num_connection_attempts = 0;  // Reset, since we have a successful read.
+			if (suppress_logging) {
+				// This was suppressed earlier, so print it out now.
+				if (encoding == Input::INPUT_ENCODING_RAW) {
+					log(INFO, "[%s] Connected to '%s', receiving raw data.",
+						   url.c_str(), url.c_str());
+				} else {
+					assert(encoding == Input::INPUT_ENCODING_METACUBE);
+					log(INFO, "[%s] Connected to '%s', receiving data.",
+						   url.c_str(), url.c_str());
+				}
+				suppress_logging = false;
+			}
+
 			process_data(buf, ret);
 			break;
 		}
@@ -455,7 +548,15 @@ void HTTPInput::do_work()
 		// or the connection just got closed.
 		// The earlier steps have already given the error message, if any.
 		if (state == NOT_CONNECTED && !should_stop()) {
-			log(INFO, "[%s] Waiting 0.2 second and restarting...", url.c_str());
+			if (!suppress_logging) {
+				log(INFO, "[%s] Waiting 0.2 seconds and restarting...", url.c_str());
+			}
+
+			if (num_connection_attempts >= 3 && !suppress_logging) {
+				log(INFO, "[%s] %d failed connection attempts, suppressing logging for one minute.",
+					url.c_str(), num_connection_attempts);
+				suppress_logging = true;
+			}
 			timespec timeout_ts;
 			timeout_ts.tv_sec = 0;
 			timeout_ts.tv_nsec = 200000000;
@@ -466,12 +567,21 @@ void HTTPInput::do_work()
 
 void HTTPInput::process_data(char *ptr, size_t bytes)
 {
-	pending_data.insert(pending_data.end(), ptr, ptr + bytes);
 	{
 		MutexLock mutex(&stats_mutex);
 		stats.bytes_received += bytes;
 	}
 
+	if (encoding == Input::INPUT_ENCODING_RAW) {
+		for (size_t i = 0; i < stream_indices.size(); ++i) {
+			servers->add_data(stream_indices[i], ptr, bytes, /*metacube_flags=*/0);
+		}
+		return;
+	}
+
+	assert(encoding == Input::INPUT_ENCODING_METACUBE);
+	pending_data.insert(pending_data.end(), ptr, ptr + bytes);
+
 	for ( ;; ) {
 		// If we don't have enough data (yet) for even the Metacube header, just return.
 		if (pending_data.size() < sizeof(metacube2_block_header)) {
@@ -543,16 +653,9 @@ void HTTPInput::process_data(char *ptr, size_t bytes)
 			for (size_t i = 0; i < stream_indices.size(); ++i) {
 				servers->set_header(stream_indices[i], http_header, stream_header);
 			}
-		} else {
-			StreamStartSuitability suitable_for_stream_start;
-			if (flags & METACUBE_FLAGS_NOT_SUITABLE_FOR_STREAM_START) {
-				suitable_for_stream_start = NOT_SUITABLE_FOR_STREAM_START;
-			} else {
-				suitable_for_stream_start = SUITABLE_FOR_STREAM_START;
-			}
-			for (size_t i = 0; i < stream_indices.size(); ++i) {
-				servers->add_data(stream_indices[i], inner_data, size, suitable_for_stream_start);
-			}
+		}
+		for (size_t i = 0; i < stream_indices.size(); ++i) {
+			servers->add_data(stream_indices[i], inner_data, size, flags);
 		}
 
 		// Consume the block. This isn't the most efficient way of dealing with things
