@@ -226,7 +226,7 @@ void UDPInput::do_work()
 		}
 		
 		for (size_t i = 0; i < stream_indices.size(); ++i) {
-			servers->add_data(stream_indices[i], packet_buf, ret, SUITABLE_FOR_STREAM_START);
+			servers->add_data(stream_indices[i], packet_buf, ret, /*metacube_flags=*/0);
 		}
 	}
 }
